import math
import random
import sys
import time

from blessings import Terminal

from . import shape, terminal

class Firework():
    '''
    A firework looks like an expanding annulus.
    (i.e. a ring, a colored circle with a black hole in the middle).
    '''
    def __init__(self):
        # Center point
        self.y, self.x = terminal.rand_coord()

        self.color = terminal.rand_color()

        # The max radius we will reach
        self.max = 2 + min(100, random.expovariate(0.2))

        # How close 'outer' gets to max before inner hole appears
        self.thickness = random.triangular(0.2, 0.95)

        # Outer and inner radius
        self.outer = 0.0
        self.inner = 0.0
        # The value of 'inner' on the last frame
        self.inner_last = self.inner

    def update(self):
        if self.outer < self.max * 0.95:
            self.outer += (self.max - self.outer) * 0.08
        if self.outer > self.max * self.thickness:
            self.inner_last = self.inner
            self.inner += (self.max - self.inner) * 0.08

    @property
    def deleteme(self):
        return self.inner > self.outer

    def draw(self):
        sys.stdout.write(self.color)
        sys.stdout.write(terminal.render(
            shape.annulus(self.y, self.x, self.outer, self.inner)
        ))
        sys.stdout.write(terminal.terminal.on_black)
        sys.stdout.write(terminal.render(
            shape.annulus(self.y, self.x, self.inner, self.inner_last)
        ))

def animate():
    world = set()
    start = time.time()
    last = 0
    while True:
        elapsed = time.time() - start
        if elapsed - last > 0.02:
            world.add(Firework())
            last = elapsed
        for item in world:
            item.update()
        for item in [item for item in world if item.deleteme]:
            world.remove(item)
        for item in world:
            item.draw()
        sys.stdout.flush()
        time.sleep(0.01)




from docopt import docopt

from . import __doc__ as root_doc, __version__ as version

def parse(args):
    return docopt(
        root_doc.format(version=version),
        argv=args,
        version=version
    )




#!/usr/bin/env python3
from glob import glob
import os
from os.path import join
import re

from setuptools import setup, find_packages

# setup.py should not import non-stdlib modules, other than setuptools, at
# module level, since this requires them to be installed to run any setup.py
# command. (e.g. 'setup.py install' should not require installing py2exe.)

# setup.py should not import from our local source (pip needs to be able to
# import setup.py before our dependencies have been installed)


NAME = 'cbeams'


def read_description(filename):
    '''
    Read given textfile and return (2nd_para, 3rd_para to end)
    '''
    with open(filename) as fp:
        text = fp.read()
    paras = text.split('\n\n')
    return paras[1], '\n\n'.join(paras[2:])


def read_file(filename):
    with open(filename, "rt") as filehandle:
        return filehandle.read()

def find_value(source, identifier):
    '''
    Manually parse the given source, looking for lines of the form:
        <identifier> = '<value>'
    Returns the value. We do this rather than import the file directly because
    its dependencies will not be present when setuptools runs this setup.py
    before installing our dependencies, to find out what they are.
    '''
    regex =r"^%s\s*=\s*['\"]([^'\"]*)['\"]$" % (identifier,)
    match = re.search(regex, source, re.M)
    if not match:
        raise RuntimeError(
            "Can't find '%s' in source:\n%s" % (identifier, source)
        )
    return match.group(1)


def get_package_data(topdir, excluded=set()):
    retval = []
    for dirname, subdirs, files in os.walk(join(NAME, topdir)):
        for x in excluded:
            if x in subdirs:
                subdirs.remove(x)
        retval.append(join(dirname[len(NAME) + 1:], '*.*'))
    return retval


def get_data_files(dest, source):
    retval = []
    for dirname, subdirs, files in os.walk(source):
        retval.append(
            (join(dest, dirname[len(source)+1:]), glob(join(dirname, '*.*')))
        )
    return retval


def get_sdist_config():
    description, long_description = read_description('README.rst')

    return dict(
        name=NAME,
        version=find_value(read_file(join(NAME, '__init__.py')), '__version__'),
        description=description,
        long_description=long_description,
        url='http://pypi.python.org/pypi/%s/' % (NAME,),
        author='Jonathan Hartley',
        author_email='tartley@tartley.com',
        keywords='console blessings ansi terminal animation',
        entry_points = {
            'console_scripts': ['{0}={0}.__main__:main'.format(NAME)],
            'gui_scripts': [],
        },
        # Application dependencies, pinned:
        install_requires=[
            'docopt==0.6.1',
            'blessings==1.6',
        ],
        packages=find_packages(exclude=['*.tests']),
        #include_package_data=True,
        #package_data={
            #'package.subpackage': ['globs'],
            #NAME: get_package_data('data')
        #},
        #exclude_package_data={
            #'package.subpackage': ['globs']
        #},
        #data_files=[
            # ('install-dir', ['files-relative-to-setup.py']),
        #], 
        # see classifiers:
        # http://pypi.python.org/pypi?:action=list_classifiers
        classifiers=[
            #'Development Status :: 1 - Planning',
            #'Development Status :: 2 - Pre-Alpha',
            #'Development Status :: 3 - Alpha',
            'Development Status :: 4 - Beta',
            #'Development Status :: 5 - Production/Stable',
            #'Development Status :: 6 - Mature',
            #'Development Status :: 7 - Inactive',
            'Environment :: Console',
            #'Environment :: MacOS X',
            #'Environment :: No Input/Output (Daemon)',
            #'Environment :: Web Environment',
            #'Environment :: Win32 (MS Windows)',
            #'Environment :: X11 Applications',
            'Intended Audience :: Developers',
            #'Intended Audience :: End Users/Desktop',
            'License :: OSI Approved :: BSD License',
            'Natural Language :: English',
            #'Operating System :: Microsoft :: Windows :: Windows 7',
            'Operating System :: MacOS :: MacOS X',
            'Operating System :: POSIX :: Linux',
            'Operating System :: OS Independent',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: Implementation :: CPython',
            'Topic :: Games/Entertainment',
        ],
        zip_safe=True,
    )


def main():
    config = {}
    config.update(get_sdist_config())
    setup(**config)


if __name__ == '__main__':
    main()




import contextlib
import functools
import sys
import random

from blessings import Terminal

terminal = Terminal()

@contextlib.contextmanager
def restore_posn_to_bottom_on_exit():
    try:
        yield
    except KeyboardInterrupt:
        pass
    sys.stdout.write(terminal.move(terminal.height - 1, 0))

@contextlib.contextmanager
def reset_on_exit(overwrite):
    if overwrite:
        fullscreen = restore_posn_to_bottom_on_exit
        init_terminal = terminal.civis + terminal.normal
    else:
        fullscreen = terminal.fullscreen
        init_terminal = terminal.civis + terminal.normal + terminal.clear

    try:
        with fullscreen():
            sys.stdout.write(init_terminal)
            yield
    except KeyboardInterrupt:
        pass
    sys.stdout.write(terminal.cnorm + terminal.normal)


def center():
    return terminal.height // 2, terminal.width // 2


def rand_coord():
    return (
        random.randint(0, terminal.height - 1),
        random.randint(0, terminal.width - 1)
    )


def rand_color():
    assert terminal.number_of_colors > 0
    return terminal.on_color(random.randint(1, terminal.number_of_colors - 1))


def clip(strips):
    '''
    Yield a sequence of strips after clipping them to (0, 0) to (height, width).
    '''
    for y, x, length in strips:
        if not(0 <= y < terminal.height):
            continue
        if x < 0:
            length += x
            x = 0
        if length + x > terminal.width:
            length = terminal.width - x
        if length > 0:
            yield y, x, length


def render(strips):
    return ''.join(
        terminal.move(y, x) + ' ' * length
        for y, x, length in clip(strips)
    )




import collections
import itertools
import math

Strip = collections.namedtuple('Strip', 'y x length')

def _subtract_strip(strip, gap):
    return [
        Strip(strip.y, strip.x, gap.x - strip.x),
        Strip(
            strip.y,
            gap.x + gap.length,
            strip.x + strip.length - gap.x - gap.length
        ),
    ]

def _subtract_row(strips, gaps):
    assert len(strips) == 1, \
        'subtracting from complicated shapes not implemented'
    strip = strips[0]
    if len(gaps) == 0:
        return [strip]
    elif len(gaps) == 1:
        return _subtract_strip(strip, gaps[0])
    else:
        raise AssertionError('subtract of complicated holes not implemented.')

def subtract(shape, hole):
    '''
    Given two shapes, e.g. return values from 'disc' or 'annulus' below,
    return the new shape that results from subtracting one from t'other.
    '''
    return list(itertools.chain.from_iterable(
        _subtract_row(
            [strip for strip in shape if strip.y == yord],
            [strip for strip in hole  if strip.y == yord]
        )
        for yord in set(strip.y for strip in shape)
    ))


def disc(y, x, radius):
    '''
    Given a filled disc of int center (y, x) and float radius,
    returns that disc as a series of horizontal strips.
    Fractional parts of radius do make a visible difference.
    '''
    if radius < 0:
        raise ValueError('radius ({}) must be >0'.format(radius))
    if radius == 0:
        return []
    else:
        strips = []
        for yoffset in range(-math.trunc(radius), math.trunc(radius) + 1):
            height = radius + yoffset
            xoffset = math.sqrt((radius - height / 2) * 2 * height)
            strips.append(Strip(
                y + yoffset,
                x - math.trunc(xoffset),
                math.trunc(xoffset) * 2 + 1
            ))
        return strips

def annulus(y, x, outer, inner):
    '''
    Given an annulus of int center (y, x), with outer and inner float radii,
    return that shape as a series of horizontal strips.
    '''
    if outer < 0 or inner < 0:
        raise ValueError(
            'outer ({}) and inner ({}) must be >= 0'.format(outer, inner)
        )
    if outer < inner:
        raise ValueError(
            'outer ({}) should be >= inner ({})'.format(outer, inner)
        )
    return subtract(disc(y, x, outer), disc(y, x, inner))

